package ru.dbotthepony.kstarbound.client.render.entity

import ru.dbotthepony.kstarbound.PIXELS_IN_STARBOUND_UNITf
import ru.dbotthepony.kstarbound.client.ClientChunk
import ru.dbotthepony.kstarbound.client.gl.GLStateTracker
import ru.dbotthepony.kstarbound.client.gl.vertex.QuadTransformers
import ru.dbotthepony.kstarbound.client.gl.vertex.quadRotatedZ
import ru.dbotthepony.kstarbound.client.render.bind
import ru.dbotthepony.kstarbound.world.entities.ItemEntity
import ru.dbotthepony.kvector.matrix.Matrix4fStack

class ItemRenderer(state: GLStateTracker, entity: ItemEntity, chunk: ClientChunk?) : EntityRenderer(state, entity, chunk) {
	private val def = entity.def
	private val textures = def.inventoryIcon?.stream()?.map { it.image.bind(state) }?.toList() ?: listOf()

	override fun render(stack: Matrix4fStack) {
		if (textures.isEmpty())
			return

		state.shaderVertexTexture.use()
		state.shaderVertexTexture.transform.set(stack.last)
		state.activeTexture = 0
		state.shaderVertexTexture["_texture"] = 0

		for (texture in textures) {
			texture.bind()

			val builder = state.flat2DTexturedQuads.small

			builder.begin()

			val width = (texture.width / PIXELS_IN_STARBOUND_UNITf) / 2f
			val height = (texture.height / PIXELS_IN_STARBOUND_UNITf) / 2f

			builder.quadRotatedZ(-width, -height, width, height, 5f, 0f, 0f, entity.movement.angle, texture.transformer)

			builder.upload()
			builder.draw()
		}
	}
}
