package ru.dbotthepony.kstarbound.defs

import ru.dbotthepony.kstarbound.io.json.FactoryAdapter

interface IThingWithDescription {
	/**
	 * Краткое описание штуки. Несмотря на то, что название свойства подразумевает "описание",
	 * на самом деле данное поле отвечает за название штуки.
	 *
	 * Примеры:
	 * * Microwave Oven
	 * * Copper Ore
	 * * Poptop
	 */
	val shortdescription: String

	/**
	 * Полное описание штуки. Оно отображается игроку, когда последний наводит курсор на штуку.
	 *
	 * Примеры:
	 * * A microwave. For when you're hungry enough to nuke your food.
	 * * Copper ore. Can be used for smelting.
	 * * The Poptop hums beautifully to confuse its prey.
	 */
	val description: String

	/**
	 * Полное описание штуки для определённых рас
	 *
	 * Пример для Microwave Oven:
	 * Apex: 			A type of oven.
	 * Avian: 			A bizarre cooking device.
	 * Floran: 			Floran likess raw meat, sssometimes cooked meat is good too.
	 * Glitch: 			Irked. It is encrusted with spattered food, who left it in this state?
	 * Human: 			A microwave. Gotta get me some jacket potatoes.
	 * Hylotl: 			A strange, spinning oven.
	 * Novakid: 		This lil' rotatin' oven cooks food at speed!
	 */
	val racialDescription: Map<String, String> get() = mapOf()

	/**
	 * Кратное описание штуки для определённых рас
	 */
	val racialShortDescription: Map<String, String> get() = mapOf()
}

data class ThingDescription(
	override val shortdescription: String,
	override val description: String
) : IThingWithDescription {
	companion object {
		val ADAPTER = FactoryAdapter.Builder(ThingDescription::class,
			ThingDescription::shortdescription,
			ThingDescription::description)
			.logMisses(false)
			.build()
	}
}
