package ru.dbotthepony.kstarbound.defs.image

import com.google.gson.GsonBuilder
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.Starbound
import ru.dbotthepony.kstarbound.registerTypeAdapter

/**
 * Хранит данные (пару) вида "/example/image.png:sprite.name"
 */
data class SpriteReference(
	val image: String,
	val sprite: AtlasConfiguration.Sprite
) {
	companion object : TypeAdapter<SpriteReference>() {
		fun parse(input: String): SpriteReference {
			val grid = AtlasConfiguration.get(input.substringBefore(':'))

			return when (input.count { it == ':' }) {
				0 -> SpriteReference(input, grid.any())
				1 -> SpriteReference(input.substringBefore(':'), grid.get(input.substringAfter(':')) ?: throw NoSuchElementException("No such sprite with name ${input.substringAfter(':')} present in frame grid ${grid.name} (atlas ${input.substringBefore(':')})"))
				else -> throw IllegalArgumentException("Invalid sprite reference: $input")
			}
		}

		override fun write(out: JsonWriter, value: SpriteReference) {
			out.value(value.image + ":" + value.sprite.name)
		}

		override fun read(`in`: JsonReader): SpriteReference {
			return parse(Starbound.assetFolder(`in`.nextString()))
		}

		fun registerGson(gsonBuilder: GsonBuilder) {
			gsonBuilder.registerTypeAdapter(this)
		}
	}
}
