package ru.dbotthepony.kstarbound.defs.item

import ru.dbotthepony.kstarbound.Starbound
import ru.dbotthepony.kstarbound.defs.IThingWithDescription
import ru.dbotthepony.kstarbound.defs.image.SpriteReference
import ru.dbotthepony.kstarbound.io.json.FactoryAdapter
import ru.dbotthepony.kstarbound.io.json.ifString

interface IItemDefinition : IThingWithDescription {
	/**
	 * Внутреннее имя предмета (ID).
	 * Не путать с именем предмета!
	 *
	 * @see shortdescription
	 * @see description
	 */
	val itemName: String

	/**
	 * Цена в пикселях
	 */
	val price: Long

	/**
	 * Редкость как [ItemRarity]
	 */
	val rarity: ItemRarity

	/**
	 * Категория предмета, определяет, в какую вкладку инвентаря оно попадает
	 */
	val category: String?

	/**
	 * Иконка в инвентаре, относительный и абсолютный пути
	 */
	val inventoryIcon: List<IInventoryIcon>?

	interface IInventoryIcon {
		val image: SpriteReference
	}

	data class InventoryIcon(
		override val image: SpriteReference
	) : IInventoryIcon {
		companion object {
			val ADAPTER = FactoryAdapter.Builder(InventoryIcon::class, InventoryIcon::image).build().ifString { InventoryIcon(SpriteReference.parse(Starbound.assetFolder(it))) }
		}
	}

	/**
	 * Теги предмета
	 */
	val itemTags: List<String>

	/**
	 * При подборе предмета мгновенно заставляет игрока изучить эти рецепты крафта
	 */
	val learnBlueprintsOnPickup: List<String>

	/**
	 * Максимальное количество предмета в стопке
	 */
	val maxStack: Long

	/**
	 * snip
	 */
	val eventCategory: String?

	/**
	 * Заставляет предмет "использовать" сразу же при подборе
	 */
	val consumeOnPickup: Boolean

	/**
	 * Запускает следующие квест(ы) при подборе
	 */
	val pickupQuestTemplates: List<String>

	/**
	 * это где либо ещё применяется кроме брони?
	 */
	val tooltipKind: ItemTooltipKind

	/**
	 * Занимает ли предмет обе руки
	 */
	val twoHanded: Boolean

	/**
	 * Заставляет SAIL/прочих болтать при подборе предмета в первый раз
	 */
	val radioMessagesOnPickup: List<String>

	/**
	 * Топливо корабля
	 */
	val fuelAmount: Long?
}
