package ru.dbotthepony.kstarbound.defs.item

import ru.dbotthepony.kstarbound.defs.ThingDescription
import ru.dbotthepony.kstarbound.defs.enrollMap
import ru.dbotthepony.kstarbound.io.json.BuilderAdapter
import ru.dbotthepony.kstarbound.io.json.INativeJsonHolder
import ru.dbotthepony.kstarbound.util.NotNullVar

open class ItemPrototype : IItemDefinition, INativeJsonHolder {
	final override var shortdescription: String = "..."
	final override var description: String = "..."
	final override var itemName: String by NotNullVar()
	final override var price: Long = 0L
	final override var rarity: ItemRarity = ItemRarity.COMMON
	final override var category: String? = null
	final override var inventoryIcon: List<IItemDefinition.InventoryIcon>? = null
	final override var itemTags: List<String> = listOf()
	final override var learnBlueprintsOnPickup: List<String> = listOf()
	final override var maxStack: Long = 9999L
	final override var eventCategory: String? = null
	final override var consumeOnPickup: Boolean = false
	final override var pickupQuestTemplates: List<String> = listOf()
	final override var tooltipKind: ItemTooltipKind = ItemTooltipKind.NORMAL
	final override var twoHanded: Boolean = false
	final override var radioMessagesOnPickup: List<String> = listOf()
	final override var fuelAmount: Long? = null

	var descriptionData: ThingDescription by NotNullVar()

	var json: Map<String, Any> = mapOf()

	final override fun acceptJson(json: MutableMap<String, Any>) {
		this.json = json
	}

	open fun assemble(): IItemDefinition {
		return ItemDefinition(
			descriptionData = descriptionData,
			itemName = itemName,
			price = price,
			rarity = rarity,
			category = category,
			inventoryIcon = inventoryIcon,
			itemTags = itemTags,
			learnBlueprintsOnPickup = learnBlueprintsOnPickup,
			maxStack = maxStack,
			eventCategory = eventCategory,
			consumeOnPickup = consumeOnPickup,
			pickupQuestTemplates = pickupQuestTemplates,
			tooltipKind = tooltipKind,
			twoHanded = twoHanded,
			radioMessagesOnPickup = radioMessagesOnPickup,
			fuelAmount = fuelAmount,

			json = enrollMap(json),
		)
	}

	companion object {
		val ADAPTER = BuilderAdapter.Builder(::ItemPrototype)
			.also(::addFields)
			.build()

		fun addFields(builder: BuilderAdapter.Builder<ItemPrototype>) {
			with(builder) {
				auto(ItemPrototype::shortdescription)
				auto(ItemPrototype::description)
				auto(ItemPrototype::itemName)
				auto(ItemPrototype::price)
				auto(ItemPrototype::rarity)
				auto(ItemPrototype::category)
				autoNullableList(ItemPrototype::inventoryIcon)
				autoList(ItemPrototype::itemTags)
				autoList(ItemPrototype::learnBlueprintsOnPickup)
				auto(ItemPrototype::maxStack)
				auto(ItemPrototype::eventCategory)
				auto(ItemPrototype::consumeOnPickup)
				autoList(ItemPrototype::pickupQuestTemplates)
				auto(ItemPrototype::tooltipKind)
				auto(ItemPrototype::twoHanded)
				autoList(ItemPrototype::radioMessagesOnPickup)
				auto(ItemPrototype::fuelAmount)
				flat(ItemPrototype::descriptionData)
			}
		}
	}
}
