package ru.dbotthepony.kstarbound.io.json

import com.google.common.collect.ImmutableMap
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.Starbound

fun <T> TypeAdapter<T>.asJsonObject(valueTransformer: (T) -> T = { it }): TypeAdapter<Map<String, T>> {
	return String2ObjectAdapter(this, valueTransformer = valueTransformer)
}

class String2ObjectAdapter<T>(val adapter: TypeAdapter<T>, val valueTransformer: (T) -> T = { it }) : TypeAdapter<Map<String, T>>() {
	constructor(type: Class<T>, valueTransformer: (T) -> T = { it }) : this(LazyTypeProvider(type), valueTransformer = valueTransformer)

	override fun write(out: JsonWriter, value: Map<String, T>) {
		out.beginObject()

		for ((k, v) in value) {
			out.name(k)
			adapter.write(out, v)
		}

		out.endObject()
	}

	override fun read(reader: JsonReader): Map<String, T> {
		val builder = ImmutableMap.builder<String, T>()

		reader.beginObject()

		while (reader.peek() != JsonToken.END_OBJECT) {
			builder.put(Starbound.STRING_INTERNER.intern(reader.nextName()), valueTransformer(adapter.read(reader)))
		}

		reader.endObject()

		return builder.build()
	}
}
